
local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Bite Buddy")

----------------------------
-- Locals

-- local LP = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")
local LSM = LibStub("LibSharedMedia-3.0")
local db

local BittenSpell = GetSpellInfo(70867)
local DebugSpell = "Horn of Winter"
local Party
local allGroups = {}
local unusedGroups = {}
local usedGroups = {}

local mainBackdrop = {
	tile = false,
	insets = { left = inset, right = inset, top = inset, bottom = inset },
}
local groupBackdrop = {
	tile = false,
	insets = { left = inset, right = inset, top = inset, bottom = inset },
}

local defaults = {
	profile = {
		x = 0,
		y = 0,
		Width = 200,
		GroupWidth = 70,
		GroupHeight = 38,
		GroupsPerRow = 5,
		BGTexture = "Solid",
		Border = "IshBorder",
		BorderSize = 12,
		Font = "accid",
		FontSize = 12,
		FontFlags = "OUTLINE",
		GroupBGColor = {0.2, 0.1, 0.9, 0.3},
		GroupBGColorHover = {0.2, 0.1, 0.9, 0.8},
		GroupBorderColor = {0.5, 0.1, 0.1, 1},
		GroupBorderColorHover = {1, 0.1, 0.1, 1},
		GroupBorderColorSelected = {0, 1, 1, 1},
	}
}

----------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("BiteBuddy", defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_StartBiteBuddy")
	
	self:SendMessage("RW_RegisterCommListener", self, "BiteBuddy_TargetSelected")
	self:SendMessage("RW_RegisterCommListener", self, "BiteBuddy_TargetDeSelected")
	
	self:CreateAnchor()
	self:Reset()
	
	Party = RW:GetPlugin("Party")
	
	self:RW_StartBiteBuddy()
end

function plugin:PluginDisable()
end

function plugin:Reset()
	db = self.db.profile
	if plugin.anchor then
		local anchor = plugin.anchor
		local scale = anchor:GetEffectiveScale() / UIParent:GetEffectiveScale()
		anchor:ClearAllPoints()
		anchor:SetPoint("CENTER", UIParent, "CENTER", db.x / scale, db.y / scale)		
		anchor:SetWidth(db.Width)
	end
end

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local scale = self:GetEffectiveScale() / UIParent:GetEffectiveScale()
	
	local x, y = self:GetCenter()
	x, y = x * scale, y * scale
	
	x = x - GetScreenWidth()/2
	y = y - GetScreenHeight()/2
	
	db.x = x
	db.y = y
	
	plugin:SendMessage("RW_GUIValuesChanged")
end

function plugin:CreateAnchor()
	local anchor = RW:CreateAnchor("BiteBuddyAnchor", UIParent, "Bite Buddy")	
	anchor:SetScript("OnDragStart", dragStart)
	anchor:SetScript("OnDragStop", dragStop)
	
	self.anchor = anchor
end

local function EnterGroup(self, event)
	self:SetBackdropColor(unpack(db.GroupBGColorHover))
	self:SetBackdropBorderColor(unpack(db.GroupBorderColorHover))
end

local function LeaveGroup(self, event)
	self:SetBackdropColor(unpack(db.GroupBGColor))
	if self.biterName:GetText() == UnitName("player") then
		self:SetBackdropBorderColor(unpack(db.GroupBorderColorSelected))
	else
		self:SetBackdropBorderColor(unpack(db.GroupBorderColor))
	end
end

local function ClickGroup(self, button)
	if button == "LeftButton" then
		if self.tagged then 
			plugin:Msg(self.name:GetText() .." is allready tagged, pick someone else")
			return 
		end
		local hasBuff = UnitAura("player", DebugSpell)
		if hasBuff then
			self:SetBackdropBorderColor(unpack(db.GroupBorderColorSelected))
			plugin:SendMessage("RW_SendComm", "BiteBuddy_TargetSelected", "RAID", nil, self.name:GetText())
		end
		
	elseif button == "RightButton" then
		if self.biterName:GetText() == UnitName("player") then
			self:SetBackdropBorderColor(unpack(db.GroupBorderColor))
			plugin:SendMessage("RW_SendComm", "BiteBuddy_TargetDeSelected", "RAID", nil, self.name:GetText())
		end
	end
end

function plugin:BiteBuddy_TargetSelected(from, dist, target)
	if usedGroups[target] then
		usedGroups[target].biterName:SetText(from)
		usedGroups[target].tagged = true
	end
end

function plugin:BiteBuddy_TargetDeSelected(from, dist, target)
	if usedGroups[target] then
		usedGroups[target].biterName:SetText("")
		usedGroups[target].tagged = false
	end
end

function plugin:CreateGroup(name, class)
	local frame = table.remove(unusedGroups)
	if not frame then
		frame = CreateFrame("Frame", "BiteBuddyGroup"..(#allGroups + 1), self:CreateMainFrame())
		
		frame:SetScript("OnEnter", EnterGroup)
		frame:SetScript("OnLeave", LeaveGroup)
		frame:SetScript("OnMouseDown", ClickGroup)
		
		frame:EnableMouse(true)
		
		local str = frame:CreateFontString("BiteBuddyGroupString"..(#allGroups + 1), "ARTWORK")
		str:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
		str:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -3)
		str:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, -3)
		str:SetHeight(20)
		frame.name = str
		
		str = frame:CreateFontString("BiteBuddyGroupBiterString"..(#allGroups + 1), "ARTWORK")
		str:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
		str:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 0, 3)
		str:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 3)
		str:SetHeight(20)
		frame.biterName = str
		
		tinsert(allGroups, frame)
	end
	local c = RAID_CLASS_COLORS[class]
	frame.name:SetText(name)
	frame.name:SetTextColor(c.r, c.g, c.b)
	frame.biterName:SetText("")
	frame.biterName:SetTextColor(1, 0, 0)
	frame:SetWidth(db.GroupWidth)
	frame:SetHeight(db.GroupHeight)
	frame:Show()
	
	
	
	return frame
end

function plugin:CreateMainFrame()
	if self.frame then return self.frame end
	local frame = CreateFrame("Frame", "BiteBuddyMainFrame", UIParent)
	frame:SetWidth(db.Width)
	frame:SetHeight(20)
	
	frame:ClearAllPoints()
	frame:SetPoint("BOTTOMLEFT", self.anchor, "TOPLEFT")
	
	self.frame = frame
	
	return frame
end

function plugin:ApplySettings()
	local frame = self:CreateMainFrame()
	
	for i, group in pairs(allGroups) do
		groupBackdrop.bgFile = LSM:Fetch("background", db.BGTexture)
		groupBackdrop.edgeFile = LSM:Fetch("border", db.Border)
		groupBackdrop.edgeSize = db.BorderSize
		group:SetBackdrop(groupBackdrop)
		group:SetBackdropColor(unpack(db.GroupBGColor))
		group:SetBackdropBorderColor(unpack(db.GroupBorderColor))
	end
end

function plugin:SetLayout()
	local col, row = 0, 0
	local frame = self:CreateMainFrame()
	for name, group in pairs(usedGroups) do
		group:ClearAllPoints()
		group:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", col * db.GroupWidth, row * db.GroupHeight)
		group:SetWidth(db.GroupWidth)
		group:SetHeight(db.GroupHeight)
		col = col + 1
		if col >= db.GroupsPerRow then
			col = 0
			row = row + 1
		end
	end
end

function plugin:RW_StartBiteBuddy()
	for name, data in Party:PartyIterator() do
		local group = self:CreateGroup(name, data.class)
		group.name:SetText(name)
		usedGroups[name] = group
	end
	self:SetLayout()
	self:ApplySettings()
end

function plugin:RW_ConfigMode(message, show)
	if show then
		self.anchor:Show()
	else
		self.anchor:Hide()
	end
end

function plugin:GetOptions()
	return "Plugins", "Bite Buddy", self.GetGUI, [[Interface\Icons\inv_misc_monsterfang_01]]
end

local function changeCallback()
	plugin:Reset()
end

local function changeCallback()
	plugin:SetLayout()
	plugin:ApplySettings()
end

function plugin:GetGUI()
	local self = self or plugin
	local UIF = LibStub("AceGUI-Factory-1.0"):GetFactory("RW")
	local AceGUI = LibStub("AceGUI-3.0")
	
	local group, label, slider, check, igroup, button, color, select
	
	group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
	
	slider = UIF:Slider("Group Width", db, "GroupWidth", 20, 300, 1, changeCallback)
	slider:SetRelativeWidth(0.5)
	group:AddChild(slider)
	
	slider = UIF:Slider("Group Height", db, "GroupHeight", 20, 300, 1, changeCallback)
	slider:SetRelativeWidth(0.5)
	group:AddChild(slider)
	
	slider = UIF:Slider("Group Per Row", db, "GroupsPerRow", 1, 10, 1, changeCallback)
	slider:SetRelativeWidth(0.5)
	group:AddChild(slider)
	
	return group
end