local mod = RW:Boss("Blood-Queen Lana'thel", "Icecrown Citadel", "Icecrown Citadel", "The Crimson Hall")
local L

mod.BossIcon = [[Interface\Icons\achievement_boss_lanathel]]

------------------------------------
-- Variables
-- Pact of the Darkfallen
local wPact, mPact, tPactCD, iPact
-- Swarming Shadows
local tSwarmDur, tSwarmCD, mSwarm, wSwarm
-- Slash
local tSlashDur, tSlashCD, mSlash
-- Bite
local tEssenceDur
-- Frenzy
local tFrenzyDur, wFrenzy
-- vampires
local mVampCount
-- Air Phase
local tAirPhaseCD, tAirPhaseDur, wAirPhase

local tBerserk

-----------------------------------
-- Init

function mod:OnRegister()
	L = mod:RegisterLocale(self:GetName(), "enUS", true)
	if L then
		L.name = "Blood-Queen Lana'thel"
		
		L.whisper_bitten = "Whisper people that get bitten"
		L.whisper_message = "You have been bitten!"
		
		L.air_phase_group = "Air Phase"
		L.air_phase_cd = "Next Air Phase"
		L.air_phase_cd_opt = "Timer for next Air Phase"
		L.air_phase_dur = "Air Phase Ends"
		L.air_phase_dur_opt = "Timer for when the Air Phase ends"
		L.air_phase_warn = "Air Phase starting!"
		L.air_phase_warn_opt = "Warning when Air phase starts"
		
		L.vamp_count = "Vampire: %s x %d"
		L.vamp_count_opt = "Show vampire count message"
		
		L.frenzy = "You need to bite someone!"
		L.frenzy_opt = "Show warning when you have to bite someone"
	end
	L = mod:GetLocale()
	mod.L = L
	
	mod:SetTrigger(37955)
	mod:SetWin()
	
	local airPhaseIcon = select(3, GetSpellInfo(71772))
	
	self:AddBoolOption("WhisperBitten", L.whisper_bitten, nil, false, [[Interface\ICONS\inv_misc_enggizmos_13]])
	
	tBerserk		= self:BerserkTimer(330)
	
	tPactCD			= self:CDTimer(30, 71340)
	tSwarm			= self:DurTargetTimer(6, 71264)
	tSwarmCD		= self:CDTimer(30, 71264)
	tSlashCD		= self:CDTimer(10, 71623, nil, nil, false)
	tSlashDur		= self:DurTargetStackTimer(15, 71623, nil, nil, false)
	tAirPhaseCD		= self:CustomTimer(125, L.air_phase_cd, L.air_phase_cd_opt, airPhaseIcon, L.air_phase_group)
	tAirPhaseDur	= self:CustomTimer(15, L.air_phase_dur, L.air_phase_dur_opt, airPhaseIcon, L.air_phase_group)
	
	wPact			= self:RunawayWarning(71340)
	wSwarm			= self:RunawayWarning(71264)
	wAirPhase		= self:CustomWarning(L.air_phase_warn, L.air_phase_warn_opt, airPhaseIcon, L.air_phase_group)

	mPact			= self:TargetMessage(71340)
	mSwarm			= self:TargetMessage(71264)
	mSlash			= self:TargetStackMessage(71623, nil, nil, false)
	
	iPact			= self:Icon("SKULL", 71340, false, 10, "CROSS", "SQUARE")
	
	-- Bite
	tEssenceDur		= self:DurTimer(60, 70867)
	tFrenzyDur		= self:DurTimer(10, 70877)
	
	mVampCount		= self:CustomMessage(L.vamp_count, L.vamp_count_opt, 70867)
	wFrenzy			= self:CustomWarning(L.frenzy, L.frenzy_opt, 70877)
end

-----------------------------------------
-- Locals
local vampCount
local vamps = {}

-----------------------------------------
-- Start

function mod:OnEngage(trigger)
	self:Track("Pact", 71340)
	self:Track("Swarm", 71265, 71277, 72638, 72639, 72640)
	self:Track("Slash", 71623)
	self:Track("Essence", 70867)
	self:Track("Frenzy", 70877, 71474)
	self:Track("Fear", 73070)

	vampCount = 0
	wipe(vamps)
	
	tAirPhaseCD:Start()
	tBerserk:Start()
end

function mod:Pact(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wPact:Show()
		end
		mPact:AddDelayTarget(args.dName)
		mPact:DelayedShow(0.2)
		tPactCD:Start()
		iPact:Show(args.dName)
	
	elseif event == "SPELL_AURA_REMOVED" then
		iPact:Hide(args.dName)
	end
end

function mod:Swarm(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wSwarm:Show()
		end
		mSwarm:Show(args.dName)
		tSwarmDur:Start(args.dName)
		tSwarmCD:Start()
	end
end

function mod:Slash(event, args)
	if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" then
		mSlash:Show(args.dName, args.amount or 1)
		tSlashDur:Start(args.dName, args.amount or 1)
		tSlashCD:Start()
	
	elseif event == "SPELL_AURA_REMOVED" then
		tSlashDur:Stop(args.dName)
	end
end

function mod:Fear(event, args)
	if event == "SPELL_CAST_SUCCESS" then
		tAirPhaseCD:Start(135)
		tAirPhaseDur:Start()
		wAirPhase:Show()
	end
end

--------------------
-- Bite
function mod:Essence(event, args)
	if event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESHED" then
		if not vamps[args.dName] then
			vamps[args.dName] = true
			vampCount = vampCount + 1
			mVampCount:Show(args.dName, vampCount)
		end
		if args:IsPlayer() then
			tEssenceDur:Start()
			tFrenzyDur:Stop()
		end
		
		if self.DB.WhisperBitten then
			if self.Party.group[args.dName] then
				SendChatMessage(L.whisper_message, "WHISPER", nil, args.dName)
			end
		end
	end
end

function mod:Frenzy(event, args)
	if event == "SPELL_AURA_APPLIED" then
		if args:IsPlayer() then
			wFrenzy:Show()
			tEssenceDur:Stop()
			tFrenzyDur:Start()
		end		
	end
end

